package lyon.audio;

import futils.utils.Draw;
import gui.Evt;
import lyon.dclap.PictFrame;

import java.awt.*;

public abstract class OscopeFrame
        extends PictFrame implements DoubleDataProducer {

    private double[] doubleData;

    private boolean drawGrid = true;
    private MenuBar mb = getMenuBar();
    private Menu m = new Menu("Scope Menu");

    private MenuItem drawGrid_mi =
            new MenuItem("[d]raw grid");


    private Scrollbar sbHorzBottom =
            new Scrollbar(Scrollbar.HORIZONTAL);
    private Scrollbar sbHorzTop =
            new Scrollbar(Scrollbar.HORIZONTAL);
    private Scrollbar sbVertLeft =
            new Scrollbar(Scrollbar.VERTICAL);
    private Scrollbar sbVertRight =
            new Scrollbar(Scrollbar.VERTICAL);

    private Color gridColor = new Color(0, 255, 0);

    private Dimension dim;

    private int height,width;


    private int dx = 0;
    private int oldDx = 0;

    private int dy = 0;
    private int oldDy = 0;

    private final double
            xScaleFactors[] = {50000, 25000, 10000, 5000, 2500, 1000,
                               500, 250, 100, 50, 25, 10,
                               5, 2.5, 1, .5, .25, .1,
                               .05, .025, .01, .005, .0025, .001,
                               .0005};

    private final String xSFLabels[] = {"0.05 u", "0.1 u", "0.25 u",
                                        "0.5 u", "1 u", "2.5 u",
                                        "5 u", "10 u", "25 u",
                                        "50 u", "100 u", "250 u",
                                        "500 u", "1 m", "2.5 m",
                                        "5 m", "10 m", "25 m",
                                        "50 m", "100 m", "250 m",
                                        "500 m", "1 ", "2.5 ",
                                        "5 "};

    private final int xsfStartIndex = 14;
    private double xScaleFactor = xScaleFactors[xsfStartIndex];
    private double oldXScaleFactor = xScaleFactors[xsfStartIndex];
    private String xSFLabel = new String(xSFLabels[xsfStartIndex]);

    private final double
            yScaleFactors[] = {
                500, 200, 100, 50, 20, 10,
                5, 2, 1, .5, .2, .1};

    private final String
            ySFLabels[] = {
                "1 m", "2.5 m",
                "5 m", "10 m",
                "25 m", "50 m",
                "100 m", "250 m",
                "500 m", "1 ",
                "2.5 ", "5 "};


    private final int ysfStartIndex = 8;
    private double yScaleFactor = 1;
    private double oldYScaleFactor = 1;
    private String ySFLabel = new String(ySFLabels[ysfStartIndex]);

    public OscopeFrame(String title) {
        super(title);
        setLayout(new BorderLayout());

        m.add(drawGrid_mi);
        mb.add(m);


        add("North", sbHorzTop);
        add("South", sbHorzBottom);
        add("West", sbVertLeft);
        add("East", sbVertRight);

        setBackground(Color.white);

        openDataFile();

        display();
    }

    private void display() {
        if (doubleData != null) {
            // Set top scrollbar characteristics
            sbHorzTop.setValues(xsfStartIndex, 0, 0, 24);
            // Set right scrollbar characteristics
            sbVertRight.setValues(ysfStartIndex, 0, 0, 11);

            // Set bottom scrollbar characteristics based on sample length
            sbHorzBottom.setValues(0, 0, 0, doubleData.length);
            // Set left scrollbar characteristics
            sbVertLeft.setValues(0, 0, -300, 300);

            setSize(550, 385);
            show();
        }
    }

    private void drawXScaleLabel(Graphics g) {
        int height = 10;
        int xloc = 20;
        int yloc = 20;

        String xsfLabel = xSFLabel + "sec/div";

        int string_width = getFontMetrics(
                g.getFont()).stringWidth(xsfLabel);
        int string_height = getFontMetrics(
                g.getFont()).getHeight();

        g.clearRect(xloc, yloc, string_width, string_height);
        g.drawString(xsfLabel, xloc, height + yloc);
    }


    private void drawYScaleLabel(Graphics g) {
        int height = 10;
        int xloc = 20;
        int yloc = dim.height - 60;

        String ysfLabel = ySFLabel + "v/div";

        int string_width = getFontMetrics(
                g.getFont()).stringWidth(ysfLabel);
        int string_height = getFontMetrics(
                g.getFont()).getHeight();

        g.clearRect(xloc, yloc, string_width, string_height);
        g.drawString(ysfLabel, xloc, height + yloc);
    }


    public void setDoubleData(double[] d) {
        doubleData = d;
        display();
    }

    public void setGridColor(Color c) {
        gridColor = c;
    }

    private void drawTrace(Graphics g) {
        doubleData = getDoubleData();

        double limit = (doubleData.length * xScaleFactor);

        dim = getSize();
        height = dim.height;
        width = dim.width;

        if (drawGrid)
            Draw.grid(20, g, gridColor);


        if (limit > width) limit = (double) width;

        int j = dx;

        done:
        for (double i = 0; i < limit; i += xScaleFactor) {
            if (j >= (doubleData.length - 1))
                break done;
            g.drawLine(
                    (int) i,
                    (int) (height - (
                    (doubleData[j] * yScaleFactor)
                    * height / 4 + height / 2)) - dy,

                    (int) (i + xScaleFactor),

                    (int) (height - (
                    (doubleData[j + 1] * yScaleFactor)
                    * height / 4 + height / 2)) - dy);

            j++;

        }

        drawXScaleLabel(g);
        drawYScaleLabel(g);

    }

    public void paint(Graphics g) {
        drawTrace(g);
    }


    public boolean handleEvent(Event e) {
        if (Evt.match(e, drawGrid_mi)) {
            drawGrid = !drawGrid;
            return true;
        }
        if (e.target == sbHorzTop) {
            int i = sbHorzTop.getValue();
            xScaleFactor = xScaleFactors[i];

            if (xScaleFactor != oldXScaleFactor) {
                xSFLabel = xSFLabels[i];
                repaint();
                oldXScaleFactor = xScaleFactor;
            }

            return true;
        }


        if (e.target == sbHorzBottom) {
            dx = sbHorzBottom.getValue();

            if (dx != oldDx) {
                repaint();
                oldDx = dx;
            }

            return true;
        }


        if (e.target == sbVertLeft) {
            dy = sbVertLeft.getValue();

            if (dy != oldDy) {
                repaint();
                oldDy = dy;
            }

            return true;
        }

        if (e.target == sbVertRight) {
            int i = sbVertRight.getValue();
            yScaleFactor = yScaleFactors[i];

            if (yScaleFactor != oldYScaleFactor) {
                ySFLabel = ySFLabels[i];
                repaint();
                oldYScaleFactor = yScaleFactor;
            }

            return true;
        }
        return super.handleEvent(e);


    }
}