package observers;
import java.awt.*;
import java.util.*;
//
// IntTextField - a TextField that reads in integer values and
// updates an Observable int with the new value.  This class
// is both a "view" of the Observable int, since it displays
// its current value, and a "controller" since it updates the
// value.
public class DoubleTextField extends TextField implements Observer
{
     private ObservableDouble value;
     public DoubleTextField(ObservableDouble theDouble)
     {
// Initialize the field to the current value, allow 3 input columns
          super(""+theDouble.getValue(), 3);
          value = theDouble;
          value.addObserver(this);     // Express interest in value
     }
// The action for the text field is called whenever someone presses "return"
// We'll try to convert the string in the field to an integer, and if
// successful, update the observable int.
     public boolean action(Event evt, Object whatAction)
     {
          Double Str;          // to be converted from a string
          try {     // The conversion can throw an exception
               Str = new Double(getText());
// If we get here, there was no exception, update the observable
               value.setValue(Str.doubleValue());
          } catch (Exception oops) {
// We just ignore the exception
          }
          return true;
     }
// The update action is called whenever the observable int's value changes.
// We just update the text in the field with the new int value
     public void update(Observable obs, Object arg)
     {
          setText(""+value.getValue());
     }
}